package org.netkernel.json.core.endpoint;

import java.io.InputStream;
import java.util.Iterator;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.netkernel.json.core.util.HDS2Utils;
import org.netkernel.layer0.meta.impl.SourcedArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.representation.ByteArrayRepresentation;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.representation.impl.HDSBuilder;
import org.netkernel.mod.hds.HDSFactory;
import org.netkernel.mod.hds.IHDSDocument;
import org.netkernel.mod.hds.IHDSMutator;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

public class JSONToHDS extends StandardAccessorImpl
{
	public static final String ARRAY_INDICATOR_SUFFIX=JSONFromHDS.ARRAY_INDICATOR_SUFFIX;
	
	public JSONToHDS()
	{	declareThreadSafe();
		declareArgument(new SourcedArgumentMetaImpl("operand","input JSON Object",null,new Class[]{JSONObject.class,JSONArray.class}));
		declareArgument(new SourcedArgumentMetaImpl("operator","configuration",null,new Class[]{IHDSNode.class}));
		declareSourceRepresentation(IHDSNode.class);
		declareSourceRepresentation(IHDSDocument.class);
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		
		String addRootElement=null;
		boolean convertToString=false;
		if (aContext.getThisRequest().argumentExists("operator"))
		{	IHDSNode operator=aContext.source("arg:operator",IHDSNode.class);
			String addRootElementString=(String)operator.getFirstValue("/*/addRootElement");
			if (addRootElementString!=null && addRootElementString.length()>0)
			{	addRootElement=addRootElementString;
			}
			Object convertToStringString=operator.getFirstValue("/*/convertToString");
			if (convertToStringString!=null && (convertToStringString.equals(Boolean.TRUE) || convertToStringString.equals("true")))
			{	convertToString=true;
			}
		}
		
		
		Object rawOperand=aContext.source("arg:operand");
		Class required=aContext.getThisRequest().getRepresentationClass();
		if (rawOperand instanceof IReadableBinaryStreamRepresentation && required==IHDSDocument.class)
		{	InputStream is=((IReadableBinaryStreamRepresentation)rawOperand).getInputStream();
			IHDSMutator m=HDSFactory.newDocument();
			if (addRootElement!=null)
			{	m.pushNode(addRootElement);
			}
			HDS2Utils.parseHDS(m,is,convertToString);
			aContext.createResponseFrom(m.toDocument(false));
		}
		else
		{	if (! (rawOperand instanceof JSONObject))
			{	rawOperand=aContext.transrept(rawOperand, JSONObject.class);
			}
			JSONObject json=(JSONObject)rawOperand;
			
			HDSBuilder b=new HDSBuilder();
			if (addRootElement!=null)
			{	b.pushNode(addRootElement);
			}
			recurseJSONToHDS(b,json,convertToString);
			aContext.createResponseFrom(b.getRoot());
		}
	}
	
	private static void recurseJSONToHDS(HDSBuilder b, JSONObject j, boolean aConvertToString) throws JSONException
	{	for (Iterator<String> i=j.keys(); i.hasNext(); )
		{	String key=i.next();
			Object object=j.get(key);
			writeJSONToHDS(b,key,object,aConvertToString);
		}
	}
	
	private static void recurseJSONArrayToHDS(HDSBuilder b, String name, JSONArray j, boolean aConvertToString) throws JSONException
	{	int l=j.length();
		for (int i=0; i<l; i++)
		{	Object object=j.get(i);
			writeJSONToHDS(b,name,object,aConvertToString);
		}
	}
	
	private static void writeJSONToHDS(HDSBuilder b,String key, Object object,boolean aConvertToString) throws JSONException
	{	if (object instanceof JSONObject)
		{	b.pushNode(key);
			recurseJSONToHDS(b,(JSONObject)object,aConvertToString);
			b.popNode();
		}
		else if (object instanceof JSONArray)
		{	b.pushNode(key+ARRAY_INDICATOR_SUFFIX);
			recurseJSONArrayToHDS(b,key,(JSONArray)object,aConvertToString);
			b.popNode();
		}
		else if (object instanceof Number || object instanceof String || object instanceof Boolean)
		{	if (aConvertToString)
			{	b.addNode(key,object.toString());
			}
			else
			{	b.addNode(key,object);
			}
		}
		else if (object==JSONObject.NULL)
		{	b.addNode(key,null);
		}
	}	
}
